@@ -1,10 +1,8 @@
 """Gosper's algorithm for hypergeometric summation. """
 
 from sympy.core import S, Dummy, symbols
-from sympy.core.compatibility import is_sequence
 from sympy.polys import Poly, parallel_poly_from_expr, factor
-from sympy.solvers import solve
-from sympy.simplify import hypersimp
+from sympy.utilities.iterables import is_sequence
 
 
 def gosper_normal(f, g, n, polys=True):
@@ -94,20 +92,21 @@ def gosper_term(f, n):
     .. math::
         s_n = \sum_{k=0}^{n-1} f_k
 
-    and `f_k` doesn't depend on `n`. Returns a hypergeometric
+    and `f_k` does not depend on `n`. Returns a hypergeometric
     term `g_n` such that `g_{n+1} - g_n = f_n`.
 
     Examples
     ========
 
     >>> from sympy.concrete.gosper import gosper_term
-    >>> from sympy.functions import factorial
+    >>> from sympy import factorial
     >>> from sympy.abc import n
 
     >>> gosper_term((4*n + 1)*factorial(n)/factorial(2*n + 1), n)
     (-n - 1/2)/(n + 1/4)
 
     """
+    from sympy.simplify import hypersimp
     r = hypersimp(f, n)
 
     if r is None:
@@ -144,6 +143,7 @@ def gosper_term(f, n):
     x = Poly(coeffs, n, domain=domain)
     H = A*x.shift(1) - B*x - C
 
+    from sympy.solvers.solvers import solve
     solution = solve(H.coeffs(), coeffs)
 
     if solution is None:
@@ -173,15 +173,15 @@ def gosper_sum(f, k):
     .. math ::
         s_n = \sum_{k=0}^{n-1} f_k
 
-    and `f(n)` doesn't depend on `n`, returns `g_{n} - g(0)` where
-    `g_{n+1} - g_n = f_n`, or ``None`` if `s_n` can not be expressed
+    and `f(n)` does not depend on `n`, returns `g_{n} - g(0)` where
+    `g_{n+1} - g_n = f_n`, or ``None`` if `s_n` cannot be expressed
     in closed form as a sum of hypergeometric terms.
 
     Examples
     ========
 
     >>> from sympy.concrete.gosper import gosper_sum
-    >>> from sympy.functions import factorial
+    >>> from sympy import factorial
     >>> from sympy.abc import n, k
 
     >>> f = (4*k + 1)*factorial(k)/factorial(2*k + 1)
