@@ -1,7 +1,17 @@
-from sympy import (
-    sqrt, Derivative, symbols, collect, Function, factor, Wild, S,
-    collect_const, log, fraction, I, cos, Add, O,sin, rcollect,
-    Mul, Pow, radsimp, diff, root, Symbol, Rational, exp, Abs)
+from sympy.core.add import Add
+from sympy.core.function import (Derivative, Function, diff)
+from sympy.core.mul import Mul
+from sympy.core.numbers import (I, Rational)
+from sympy.core.power import Pow
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, Wild, symbols)
+from sympy.functions.elementary.complexes import Abs
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.miscellaneous import (root, sqrt)
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.polys.polytools import factor
+from sympy.series.order import O
+from sympy.simplify.radsimp import (collect, collect_const, fraction, radsimp, rcollect)
 
 from sympy.core.expr import unchanged
 from sympy.core.mul import _unevaluated_Mul as umul
@@ -158,7 +168,7 @@ def test_radsimp_issue_3214():
 
 
 def test_collect_1():
-    """Collect with respect to a Symbol"""
+    """Collect with respect to Symbol"""
     x, y, z, n = symbols('x,y,z,n')
     assert collect(1, x) == 1
     assert collect( x + y*x, x ) == x * (1 + y)
@@ -174,6 +184,11 @@ def test_collect_1():
     # symbols can be given as any iterable
     expr = x + y
     assert collect(expr, expr.free_symbols) == expr
+    assert collect(x*exp(x) + sin(x)*y + sin(x)*2 + 3*x, x, exact=None
+        ) == x*exp(x) + 3*x + (y + 2)*sin(x)
+    assert collect(x*exp(x) + sin(x)*y + sin(x)*2 + 3*x + y*x +
+        y*x*exp(x), x, exact=None
+        ) == x*exp(x)*(y + 1) + (3 + y)*x + (y + 2)*sin(x)
 
 
 def test_collect_2():
@@ -429,7 +444,8 @@ def test_issue_5615():
 
 
 def test_issue_5933():
-    from sympy import Polygon, RegularPolygon, denom
+    from sympy.geometry.polygon import (Polygon, RegularPolygon)
+    from sympy.simplify.radsimp import denom
     x = Polygon(*RegularPolygon((0, 0), 1, 5).vertices).centroid.x
     assert abs(denom(x).n()) > 1e-12
     assert abs(denom(radsimp(x))) > 1e-12  # in case simplify didn't handle it
