@@ -1,7 +1,14 @@
-from sympy import (Add, Matrix, Mul, S, symbols, Eq, pi, factorint, oo,
-                   powsimp, Rational)
+from sympy.core.add import Add
+from sympy.core.mul import Mul
+from sympy.core.numbers import (Rational, oo, pi)
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.core.symbol import symbols
+from sympy.matrices.dense import Matrix
+from sympy.ntheory.factor_ import factorint
+from sympy.simplify.powsimp import powsimp
 from sympy.core.function import _mexpand
-from sympy.core.compatibility import ordered
+from sympy.core.sorting import default_sort_key, ordered
 from sympy.functions.elementary.trigonometric import sin
 from sympy.solvers.diophantine import diophantine
 from sympy.solvers.diophantine.diophantine import (diop_DN,
@@ -17,7 +24,6 @@ from sympy.solvers.diophantine.diophantine import (diop_DN,
     _diop_ternary_quadratic_normal, _nint_or_floor,
     _odd, _even, _remove_gcd, _can_do_sum_of_squares, DiophantineSolutionSet, GeneralPythagorean,
     BinaryQuadratic)
-from sympy.utilities import default_sort_key
 
 from sympy.testing.pytest import slow, raises, XFAIL
 from sympy.utilities.iterables import (
@@ -195,7 +201,7 @@ def test_quadratic_non_perfect_slow():
 def test_DN():
     # Most of the test cases were adapted from,
     # Solving the generalized Pell equation x**2 - D*y**2 = N, John P. Robertson, July 31, 2004.
-    # http://www.jpr2718.org/pell.pdf
+    # https://web.archive.org/web/20160323033128/http://www.jpr2718.org/pell.pdf
     # others are verified using Wolfram Alpha.
 
     # Covers cases where D <= 0 or D > 0 and D is a square or N = 0
@@ -569,7 +575,6 @@ def test_diophantine():
     assert diophantine((y**2-x), t) == {(t**2, -t)}
 
 
-
 def test_general_pythagorean():
     from sympy.abc import a, b, c, d, e
 
@@ -617,12 +622,23 @@ def test_diop_general_sum_of_squares_quick():
     raises(NotImplementedError, lambda: classify_diop(-eq))
 
 
+def test_issue_23807():
+    # fixes recursion error
+    eq = x**2 + y**2 + z**2 - 1000000
+    base_soln = {(0, 0, 1000), (0, 352, 936), (480, 600, 640), (24, 640, 768), (192, 640, 744),
+                 (192, 480, 856), (168, 224, 960), (0, 600, 800), (280, 576, 768), (152, 480, 864),
+                 (0, 280, 960), (352, 360, 864), (424, 480, 768), (360, 480, 800), (224, 600, 768),
+                 (96, 360, 928), (168, 576, 800), (96, 480, 872)}
+
+    assert diophantine(eq) == base_soln
+
+
 def test_diop_partition():
     for n in [8, 10]:
         for k in range(1, 8):
             for p in partition(n, k):
                 assert len(p) == k
-    assert [p for p in partition(3, 5)] == []
+    assert list(partition(3, 5)) == []
     assert [list(p) for p in partition(3, 5, 1)] == [
         [0, 0, 0, 0, 3], [0, 0, 0, 1, 2], [0, 0, 1, 1, 1]]
     assert list(partition(0)) == [()]
@@ -652,7 +668,7 @@ def test_sum_of_three_squares():
 
 
 def test_sum_of_four_squares():
-    from random import randint
+    from sympy.core.random import randint
 
     # this should never fail
     n = randint(1, 100000000000000)
